@@ -1152,13 +1152,17 @@ static struct qmi_request *find_control_request(struct qmi_qmux_device *qmux,
 
 static void __qmux_discovery_finished(struct qmi_qmux_device *qmux)
 {
+	qmi_qmux_device_discover_func_t func = qmux->discover.func;
+	void *user_data = qmux->discover.user_data;
+	qmi_destroy_func_t destroy = qmux->discover.destroy;
+
 	l_timeout_remove(qmux->discover.timeout);
-	qmux->discover.func(qmux->discover.user_data);
-
-	if (qmux->discover.destroy)
-		qmux->discover.destroy(qmux->discover.user_data);
-
 	memset(&qmux->discover, 0, sizeof(qmux->discover));
+
+	func(user_data);
+
+	if (destroy)
+		destroy(user_data);
 }
 
 static void qmux_sync_callback(struct qmi_request *req, uint16_t message,
@@ -2307,7 +2311,7 @@ bool qmi_result_get_uint8(struct qmi_result *result, uint8_t type,
 		return false;
 
 	ptr = tlv_get(result->data, result->length, type, &len);
-	if (!ptr)
+	if (!ptr || len != sizeof(uint8_t))
 		return false;
 
 	if (value)
@@ -2326,7 +2330,7 @@ bool qmi_result_get_int16(struct qmi_result *result, uint8_t type,
 		return false;
 
 	ptr = tlv_get(result->data, result->length, type, &len);
-	if (!ptr)
+	if (!ptr || len != sizeof(int16_t))
 		return false;
 
 	memcpy(&tmp, ptr, 2);
@@ -2347,7 +2351,7 @@ bool qmi_result_get_uint16(struct qmi_result *result, uint8_t type,
 		return false;
 
 	ptr = tlv_get(result->data, result->length, type, &len);
-	if (!ptr)
+	if (!ptr || len != sizeof(uint16_t))
 		return false;
 
 	memcpy(&tmp, ptr, 2);
@@ -2369,7 +2373,7 @@ bool qmi_result_get_uint32(struct qmi_result *result, uint8_t type,
 		return false;
 
 	ptr = tlv_get(result->data, result->length, type, &len);
-	if (!ptr)
+	if (!ptr || len != sizeof(uint32_t))
 		return false;
 
 	memcpy(&tmp, ptr, 4);
@@ -2391,7 +2395,7 @@ bool qmi_result_get_uint64(struct qmi_result *result, uint8_t type,
 		return false;
 
 	ptr = tlv_get(result->data, result->length, type, &len);
-	if (!ptr)
+	if (!ptr || len != sizeof(uint64_t))
 		return false;
 
 	memcpy(&tmp, ptr, 8);
