@@ -1,22 +1,8 @@
 /*
+ * oFono - Open Source Telephony
+ * Copyright (C) 2008-2011  Intel Corporation
  *
- *  oFono - Open Source Telephony
- *
- *  Copyright (C) 2008-2011  Intel Corporation. All rights reserved.
- *
- *  This program is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License version 2 as
- *  published by the Free Software Foundation.
- *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
- *
+ * SPDX-License-Identifier: GPL-2.0-only
  */
 
 #ifdef HAVE_CONFIG_H
@@ -219,7 +205,6 @@ static void stktest_post_online(struct ofono_modem *modem)
 
 static struct ofono_modem_driver stktest_driver = {
 	.modem_type	= OFONO_MODEM_TYPE_TEST,
-	.name		= "stktest",
 	.probe		= stktest_probe,
 	.remove		= stktest_remove,
 	.enable		= stktest_enable,
@@ -230,14 +215,10 @@ static struct ofono_modem_driver stktest_driver = {
 	.post_online	= stktest_post_online,
 };
 
+OFONO_MODEM_DRIVER_BUILTIN(stktest, &stktest_driver)
+
 static int stktest_init(void)
 {
-	int err;
-
-	err = ofono_modem_driver_register(&stktest_driver);
-	if (err < 0)
-		return err;
-
 	stktest = ofono_modem_create("stktest", "stktest");
 	ofono_modem_register(stktest);
 
@@ -247,7 +228,6 @@ static int stktest_init(void)
 static void stktest_exit(void)
 {
 	ofono_modem_remove(stktest);
-	ofono_modem_driver_unregister(&stktest_driver);
 }
 
 OFONO_PLUGIN_DEFINE(stktest, "STK End-to-End tester driver", VERSION,
